
#include "ListTest.h"

#define ENTRIES_NUM 16		// ,    

/*       ""  */
UINT32              evcode_base;

/*  .        */
const char app_name[APP_NAME_LEN] = "HelloWorld"; 

/*     state- */

/*    HW_STATE_ANY (   state) */
const EVENT_HANDLER_ENTRY_T any_state_handlers[] =
{  
    /* GUI-     */
    { EV_REVOKE_TOKEN,              APP_HandleUITokenRevoked },
    { STATE_HANDLERS_END,           NULL           },
};

const EVENT_HANDLER_ENTRY_T init_state_handlers[] =
{
    /*   ,      UI */
    { EV_GRANT_TOKEN,               HandleUITokenGranted },
    { STATE_HANDLERS_END,           NULL           },
};

EVENT_HANDLER_ENTRY_T main_state_handlers[] =
{
    { EV_DIALOG_DONE,               HelloWorldExit },
    { EV_REQUEST_LIST_ITEMS,        HandleListReq  },
    /* STATE_HANDLERS_RESERVED ,   - ,  
          ,      
        . LdrInitEventHandlersTbl, LdrFindEventHandlerTbl */
    { STATE_HANDLERS_RESERVED,      HandleAction   },
    { EV_DONE,                      HelloWorldExit },
    { STATE_HANDLERS_END,           NULL           },
};

/*   ,    -  .
      ,   enum-e */
static const STATE_HANDLERS_ENTRY_T state_handling_table[] =
{
    { HW_STATE_ANY,               // State
      NULL,                       //    state
      NULL,                       //    state
      any_state_handlers          //   
    },

    { HW_STATE_INIT,
      NULL,
      NULL,
      init_state_handlers
    },
    
    { HW_STATE_MAIN,
      MainStateEnter,
      NULL,
      main_state_handlers
    }
};

/*      elfpack- */
UINT32 LdrInitEventHandlersTbl( EVENT_HANDLER_ENTRY_T *tbl,  UINT32 *base )
{
	UINT32			i=0;
	while( tbl[i].code!=STATE_HANDLERS_END )
	{
		if(tbl[i].code==STATE_HANDLERS_RESERVED)	
		{
			tbl[i].code = (*base)++;
		}
		i++;
	}
	return *base;
}

UINT32 LdrFindEventHandlerTbl( EVENT_HANDLER_ENTRY_T *tbl,  EVENT_HANDLER_T *hfn )
{
	UINT32			i=0;
	while( tbl[i].code!=STATE_HANDLERS_END )
	{
		if(tbl[i].hfunc==hfn)	
		{
			return tbl[i].code;
		}
		i++;
	}

	return 0;
}


/*  entry   ,      */
/* file_uri -    ( argv[0]) */
/* param -   (  argv) */
/* reserve -   eventcode- ( 64 )*/
UINT32 Register( char* file_uri,  char* param,  UINT32 reserve )
{
    UINT32 status = RESULT_OK;
	
    evcode_base = reserve;    //  reserve
	//    ,   STATE_HANDLERS_RESERVED
	LdrInitEventHandlersTbl( main_state_handlers, &evcode_base );
    

/*   */
    status = APP_Register( &evcode_base,            //      
                           1,                       // - ,     
                           state_handling_table,    //  
                           HW_STATE_MAX,            //  
                           (void*)HelloWorldStart );   // ,   
                   
    UtilLogStringData(" *** ELF *** HelloWorld: Register status %d", status);
    
    LdrStartApp(evcode_base);   //  
    
    return 1;   //    ,  
}


/*      */
UINT32 HelloWorldStart( EVENT_STACK_T *ev_st,  REG_ID_T reg_id,  UINT32 param2 )//void *reg_hdl )
{
    APP_HELLOWORLD_T     *app = NULL;
    UINT32 status = RESULT_OK;
    
    UtilLogStringData(" *** ELF *** HelloWorld: HelloWorldStart");

    /* ,    ? */
    if( AFW_InquireRoutingStackByRegId( reg_id ) == RESULT_OK )
        return RESULT_FAIL; //     

    /*     */
    app = (APP_HELLOWORLD_T*)APP_InitAppData( (void *)APP_HandleEvent, //     GUI
                                              sizeof(APP_HELLOWORLD_T), //   
                                              reg_id,
                                              0, 1,
                                              1,
                                              1, 1, 0 );

	/*   */
	status = InitResources();
	UtilLogStringData(" *** ELF *** HelloWorld: InitResources status = %d", status);

    UtilLogStringData(" *** ELF *** HelloWorld: APP_InitAppData 0x%X", app);
    status = APP_Start( ev_st,
                        &app->apt,
                        HW_STATE_INIT, //  
                        state_handling_table,
                        HelloWorldExit,
                        app_name,
                        0 );

    UtilLogStringData(" *** ELF *** HelloWorld: APP_Start 0x%X", status);

    return RESULT_OK;
}


/*     */
UINT32 HelloWorldExit( EVENT_STACK_T *ev_st,  void *app )
{
    UINT32  status;

	/*   */
	RemoveResources();

    /*    */
    status = APP_Exit( ev_st, app, 0 );

    /*   */
    LdrUnloadELF(&Lib); // &Lib    

    return status;
}

/*    main state */
UINT32 MainStateEnter( EVENT_STACK_T *ev_st,  void *app,  ENTER_STATE_TYPE_T type )
{
    APPLICATION_T           *papp = (APPLICATION_T*) app;
    SU_PORT_T               port = papp->port;
    UIS_DIALOG_T            dialog = 0;
	UINT32					starting_num;
	ACTIONS_T				action_list; // 

	if(type!=ENTER_STATE_ENTER) return RESULT_OK;

	InitDlgActions( &action_list );
	
	dialog = UIS_CreateList( &port,
							 0,
							 ENTRIES_NUM,
							 0,
							 &starting_num,
							 0,
							 2,
							 &action_list,
							 Resources[RES_LIST_CAPTION] );
  
    UtilLogStringData(" *** ELF *** HelloWorld: MainStateEnter 0x%X", dialog);
    
    if(dialog == 0) return RESULT_FAIL;

    papp->dialog = dialog;

	//   ,     starting_num 
	SendListItems(ev_st, app, 1, starting_num);

    return RESULT_OK;
}

UINT32 HandleUITokenGranted( EVENT_STACK_T *ev_st,  void *app )
{
    APPLICATION_T           *papp = (APPLICATION_T*) app;
    UINT32                  status;

    //    
    status = APP_HandleUITokenGranted( ev_st, app );

    //   ,   state
    if( (status == RESULT_OK) && (papp->token_status == 2) )
    {
        status = APP_UtilChangeState( HW_STATE_MAIN, ev_st, app );
    }

    return status;
}


/* -  EV_REQUEST_LIST_ITEMS -   List    
          */
UINT32 HandleListReq( EVENT_STACK_T *ev_st,  void *app )
{
	APPLICATION_T			*papp = (APPLICATION_T*) app;
	EVENT_T					*event;
	UINT32					start, num;

	//      "",      
	if( !papp->focused ) return RESULT_OK;

	//      
	event = AFW_GetEv( ev_st );

	start = event->data.list_items_req.begin_idx;
	num = event->data.list_items_req.count;

	//   ,     - ,      
	APP_ConsumeEv( ev_st, app );
	
	return SendListItems( ev_st, app, start, num );
}

UINT32 HandleAction( EVENT_STACK_T *ev_st,  void *app )
{
	APP_ConsumeEv( ev_st, app );
	
    UtilLogStringData(" *** ELF *** HelloWorld: HandleAction");
	/*     -  */
	return APP_Exit( ev_st, app, 0);
	
}


/*            */
UINT32 SendListItems( EVENT_STACK_T *ev_st,  void *app, UINT32 start, UINT32 num)
{
	APPLICATION_T			*papp = (APPLICATION_T*) app;
	LIST_ENTRY_T			*plist; //    
	UINT32					i, index, status=RESULT_OK;
    
    if( num==0 ) return RESULT_FAIL; //  ,   

	plist = (LIST_ENTRY_T*) suAllocMem( sizeof(LIST_ENTRY_T)*num, NULL ); // malloc
	if( plist==NULL ) return RESULT_FAIL;

	//  
	for( index=0, i=start; (i<start+num) && (i<=ENTRIES_NUM); i++, index++)
	{
		plist[index].editable = FALSE;			//  
		plist[index].content.static_entry.unk6 = 1;

		//        
		UIS_MakeContentFromString( "Cs0", &(plist[index].content.static_entry.text), 0x010001CB+i-1 );
	}

	//       ,    
	status = APP_UtilAddEvUISListData( ev_st, app, 0,
									   start, num,
									   FBF_LEAVE,
									   sizeof(LIST_ENTRY_T)*num,
									   plist );
	if( status!=RESULT_FAIL )
	{
		//        
		status = UIS_HandleEvent(papp->dialog, ev_st);
	}

	suFreeMem(plist);

	return status;

}

/*           */
UINT32 InitResources( )
{
    RES_ACTION_LIST_ITEM_T		action;
    UINT32						status;

	const WCHAR list_caption[] = L"ListTest";

	/*     */
    status = DRM_CreateResource( &Resources[RES_LIST_CAPTION], RES_TYPE_STRING, (void*)list_caption, (u_strlen((WCHAR*)list_caption)+1)*sizeof(WCHAR) );
	if( status!=RESULT_OK ) return status;

	/*    */
    action.softkey_label = 0;			//      
    action.list_label = 0x01001B77;		//     
    action.softkey_priority = 0;
    action.list_priority = 1;			//     , ..        ,   0,     
    action.isExit = FALSE;
    action.sendDlgDone = FALSE;

    status = DRM_CreateResource( &Resources[RES_ACTION1], RES_TYPE_ACTION, (void*)&action, sizeof(RES_ACTION_LIST_ITEM_T));
	if( status!=RESULT_OK ) DRM_ClearResource( Resources[RES_LIST_CAPTION] );
    return status;
}

/*        */
UINT32 RemoveResources( )
{
	UINT32				status = RESULT_OK;
	UINT32				i;

	for(i=0;i<RES_MAX;i++)
	{
		status |= DRM_ClearResource( Resources[i] );
	}

	return status;

}

/*   */
UINT32 InitDlgActions( ACTIONS_T *action_list )
{
	/*  -           */
	
	/*    -    ,     
		      . */

	/*     (,   ) */
	/*   List       .  - ""  "",
		EV_DONE  EV_SELECT .        . */


	/*          */	
    action_list->action[0].operation = ACTION_OP_ADD; 
	/*         STATE_HANDLERS_RESERVED,        */
    action_list->action[0].event = LdrFindEventHandlerTbl( main_state_handlers, HandleAction ); //       
    action_list->action[0].action_res = Resources[RES_ACTION1]; // ,  

	/*      */
    action_list->count = 1;

    return RESULT_OK;
}

  
